/*
 * config.h
 *
 *  Created on: Sep 5, 2021
 *      Author: dukenuc
 */

#ifndef INC_CONFIG_H_
#define INC_CONFIG_H_

/* TODO: add comments to everything */

#include "stdint.h"
#include "stm32l0xx.h"

#define VENDOR_NAME_LENGTH			64
#define	PRODUCT_CODE_LENGTH			64
#define	PRODUCT_NAME_LENGTH			64
#define REVISION_LENGTH				16
#define SERIAL_NUMBER_LENGTH		64

#define DATA_EEPROM_BASE_ADDR ((uint32_t)0x08080000) /* Data EEPROM base address */
#define DATA_EEPROM_END_ADDR ((uint32_t)0x080801FF) /* Data EEPROM end address */

#define FLASH_PEKEY1				0x89ABCDEF
#define FLASH_PEKEY2				0x02030405

#define CONFIG_OK					0
#define CONFIG_ERROR				-1

#define EEPROM_OK					0
#define EEPROM_ERROR				-1
#define EEPROM_UNLOCK_ERROR			-2
#define EEPROM_LOCK_ERROR			-3
#define EEPROM_WRITE_ERROR			-4



typedef struct
{
	/* DEVICE ID */
	uint8_t vendor_name[VENDOR_NAME_LENGTH];
	uint8_t product_code[PRODUCT_CODE_LENGTH];
	uint8_t product_name[PRODUCT_NAME_LENGTH];
	uint8_t revision[REVISION_LENGTH];
	uint8_t serial_number[SERIAL_NUMBER_LENGTH];

	/* DEVICE SPECIFIC CONFIG */
	uint8_t led_on;
	uint16_t led_co2_alert_limit1;
	uint16_t led_co2_alert_limit2;
} config_t;

int8_t config_read(config_t *config);

int8_t config_write(config_t *config);

static int8_t eeprom_lock(void);
static int8_t eeprom_unlock(void);
static int8_t eeprom_program_byte(uint32_t addr, uint8_t ee_data);
static int8_t eeprom_program_halfword(uint32_t addr, uint16_t ee_data);
static int8_t eeprom_program_word(uint32_t addr, uint32_t ee_data);

#endif /* INC_CONFIG_H_ */

